@@ -1,10 +1,16 @@
-def force_list (var):
+import json
+import logging
+
+import yaml
+
+
+def force_list(var):
     if not isinstance(var, list):
         var = [var]
     return var
 
 
-def force_int (var):
+def force_int(var):
     try:
         if not isinstance(var, int):
             var = int(var)
@@ -12,7 +18,8 @@ def force_int (var):
     except:
         return None
 
-def force_float (var):
+
+def force_float(var):
     try:
         if not isinstance(var, float):
             var = float(var)
@@ -20,6 +27,7 @@ def force_float (var):
     except:
         return None
 
+
 def convert_str_to_bool(bool_str):
     if bool_str in ['true', '"true"', 'True', '"True"']:
         return True
@@ -41,3 +49,21 @@ def force_dict(obj):
     if type(obj) == list and len(obj) > 0 and type(obj[0]) == dict:
         return obj[0]
     return None
+
+
+def is_json(data) -> bool:
+    try:
+        parsed = json.loads(data)
+        return isinstance(parsed, dict)
+    except (TypeError, ValueError):
+        logging.debug(f"could not parse json data: {data}")
+        return False
+
+
+def is_yaml(data) -> bool:
+    try:
+        parsed = yaml.safe_load(data)
+        return isinstance(parsed, dict)
+    except yaml.YAMLError:
+        logging.debug(f"could not parse yaml data: {data}")
+        return False
