@@ -14,6 +14,6 @@ class AbsGoogleIAMMemberDefaultServiceAccount(BaseResourceCheck):
 
     def scan_resource_conf(self, conf):
         members_conf = conf['members'][0] if 'members' in conf else conf.get('member', [])
-        if any(re.match(DEFAULT_SA, member) for member in members_conf):
+        if any(re.match(DEFAULT_SA, str(member)) for member in members_conf):
             return CheckResult.FAILED
         return CheckResult.PASSED
