@@ -1,26 +1,26 @@
-
-from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
 from checkov.common.models.enums import CheckCategories, CheckResult
+from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
+
 
 class GKESecureBootforShieldedNodes(BaseResourceValueCheck):
     def __init__(self):
         name = "Ensure Secure Boot for Shielded GKE Nodes is Enabled"
         id = "CKV_GCP_73"
-        supported_resources = ['google_container_cluster','google_container_node_pool']
+        supported_resources = ['google_container_cluster', 'google_container_node_pool']
         categories = [CheckCategories.KUBERNETES]
         super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)
 
     def scan_resource_conf(self, conf):
         if 'node_config' in conf.keys():
-            node=conf["node_config"][0]
-            if 'shielded_instance_config' in node.keys():
-                monitor=node["shielded_instance_config"][0]
+            node = conf["node_config"][0]
+            if isinstance(node, dict) and 'shielded_instance_config' in node:
+                monitor = node["shielded_instance_config"][0]
                 if monitor["enable_secure_boot"] == [True]:
                     return CheckResult.PASSED
                 else:
                     return CheckResult.FAILED
             else:
-                #as default is true this is a pass 
+                # as default is true this is a pass
                 return CheckResult.FAILED
         else:
             return CheckResult.UNKNOWN
@@ -32,4 +32,4 @@ class GKESecureBootforShieldedNodes(BaseResourceValueCheck):
         return [True]
 
 
-check = GKESecureBootforShieldedNodes()
\ No newline at end of file
+check = GKESecureBootforShieldedNodes()
